package editor;

import game.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import javax.imageio.ImageIO;
import javax.swing.*;

import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;

public class EditPanel extends JPanel implements MouseListener, MouseMotionListener{
  public static final int EDIT_PLAYER = 0;
  public static final int EDIT_ENEMY = 1;
  public static final int EDIT_OBSTACLE = 2;
  public static final int EDIT_GOAL = 3;

  private int mode = EDIT_PLAYER;
  private boolean grid = true;

  private java.util.List<Point> players, enemies;
  private java.util.List<Obstacle> obstacles;
  private Rectangle goal;

  private boolean dragging = false;
  private Point dragStart;
  private Point dragCurrent;

  public EditPanel(){
    setPreferredSize(Game.boardSize);

    addMouseListener(this);
    addMouseMotionListener(this);

    players = new Vector<Point>();
    enemies = new Vector<Point>();
    obstacles = new Vector<Obstacle>();
    goal = new Rectangle();
  }

  public void setMode(int mode){ this.mode = mode; }

  public void saveFile(File f){
    try{
      DocumentBuilder builder =
        DocumentBuilderFactory.newInstance().newDocumentBuilder();
      Document doc = builder.newDocument();

      Element root = doc.createElement("game");
      doc.appendChild(root);

      root.setAttribute("pmax", "18");
      root.setAttribute("emax", "20");

      Element list;
      list = doc.createElement("players");
      root.appendChild(list);
      for(Point p : players){
        Element e = doc.createElement("player");
        e.setAttribute("x", Integer.toString(p.x));
        e.setAttribute("y", Integer.toString(p.y));
        list.appendChild(e);
      }

      list = doc.createElement("enemies");
      root.appendChild(list);
      for(Point p : enemies){
        Element e = doc.createElement("enemy");
        e.setAttribute("x", Integer.toString(p.x));
        e.setAttribute("y", Integer.toString(p.y));
        list.appendChild(e);
      }

      list = doc.createElement("obstacles");
      root.appendChild(list);
      for(Obstacle o : obstacles){
        Element e = doc.createElement("obstacle");
        e.setAttribute("x", Integer.toString(o.x));
        e.setAttribute("y", Integer.toString(o.y));
        e.setAttribute("r", Integer.toString(o.r));
        list.appendChild(e);
      }

      Element e = doc.createElement("goal");
      e.setAttribute("x", Integer.toString(goal.x));
      e.setAttribute("y", Integer.toString(goal.y));
      e.setAttribute("w", Integer.toString(goal.width));
      e.setAttribute("h", Integer.toString(goal.height));
      root.appendChild(e);

      DOMSource source = new DOMSource(doc);
      StreamResult result = new StreamResult(f);
      Transformer transformer = TransformerFactory.newInstance().newTransformer();
      transformer.setOutputProperty(OutputKeys.INDENT, "yes");
      transformer.transform(source, result);
    }catch(Exception e){ e.printStackTrace(); }
  }

  public void saveImage(File f){
    grid = false;
    BufferedImage image = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_ARGB);
    Graphics g = image.getGraphics();
    paint(g);
    try{ ImageIO.write(image, "png", f); }
    catch(IOException e){ e.printStackTrace(); }
    grid = true;
  }
  
  public void paintComponent(Graphics g){
    super.paintComponent(g);
    Graphics2D g2D = (Graphics2D) g;
    g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    // draw grid
    if(grid){
      int interval = 32, width = getWidth(), height = getHeight();
      for(int x = 0; x < width; x += interval) g.drawLine(x, 0, x, height);
      for(int y = 0; y < getHeight(); y += interval) g.drawLine(0, y, width, y);
    }

    int r = (int) Game.radius, r2 = (int) (Game.radius * 2);

    g.setColor(Color.black);
    g.drawRect(goal.x, goal.y, goal.width, goal.height);

    g.setColor(Color.blue);
    for(Point p : players){
      g.fillOval(p.x - r, p.y - r, r2, r2); // for now
    }

    g.setColor(Color.red);
    for(Point p : enemies){
      g.fillOval(p.x - r, p.y - r, r2, r2);
    }

    g.setColor(Color.green);
    for(Obstacle o : obstacles){
      g.fillOval(o.x - o.r, o.y - o.r, o.r * 2, o.r * 2);
    }

    if(dragging){
      int x = Math.min(dragStart.x, dragCurrent.x);
      int y = Math.min(dragStart.y, dragCurrent.y);
      int w = Math.abs(dragStart.x - dragCurrent.x);
      int h = Math.abs(dragStart.y - dragCurrent.y);
      switch(mode){
      case EDIT_OBSTACLE:
        int max = Math.max(w, h);
        g.drawOval(x, y, max, max);
        break;
      case EDIT_GOAL:
        g.drawRect(x, y, w, h);
        break;
      }
    }
  }

  public void mousePressed(MouseEvent e){
    switch(mode){
    case EDIT_OBSTACLE:
    case EDIT_GOAL:
      dragStart = e.getPoint();
    }
  }

  public void mouseReleased(MouseEvent e){
    if(dragging){
      int x = Math.min(dragStart.x, dragCurrent.x);
      int y = Math.min(dragStart.y, dragCurrent.y);
      int w = Math.abs(dragStart.x - dragCurrent.x);
      int h = Math.abs(dragStart.y - dragCurrent.y);
      switch(mode){
      case EDIT_OBSTACLE:
        int r = Math.max(w, h) / 2;
        obstacles.add(new Obstacle(x + r, y + r, r));
        break;
      case EDIT_GOAL:
        goal.setBounds(x, y, w, h);
      }
    }
    dragging = false;
    repaint();
  }

  public void mouseClicked(MouseEvent e){
    if(SwingUtilities.isLeftMouseButton(e)){
      switch(mode){
      case EDIT_PLAYER:
        players.add(e.getPoint());
        break;
      case EDIT_ENEMY:
        enemies.add(e.getPoint());
      }
    }
    else{
      switch(mode){
      case EDIT_PLAYER:
        for(ListIterator<Point> iter = players.listIterator(players.size()); iter.hasPrevious(); ){
          Point p = iter.previous();
          if(p.distance(e.getX(), e.getY()) < 10) iter.remove();
        }
        break;
      case EDIT_ENEMY:
        for(ListIterator<Point> iter = enemies.listIterator(enemies.size()); iter.hasPrevious(); ){
          if(iter.previous().distance(e.getX(), e.getY()) < 10) iter.remove();
        }
        break;
      case EDIT_OBSTACLE:
        for(ListIterator<Obstacle> iter = obstacles.listIterator(obstacles.size()); iter.hasPrevious(); ){
          Obstacle o = iter.previous();
          if(Point2D.distance(o.x, o.y, e.getX(), e.getY()) < o.r) iter.remove();
        }
        break;
      }
    }
    repaint();
  }

  public void mouseDragged(MouseEvent e){
    switch(mode){
    case EDIT_OBSTACLE:
    case EDIT_GOAL:
      dragging = true;
      dragCurrent = e.getPoint();
      repaint();
    }
  }

  public void mouseEntered(MouseEvent e){}
  public void mouseExited(MouseEvent e){}
  public void mouseMoved(MouseEvent e){}

  class Obstacle{
    int x, y, r;

    public Obstacle(int x, int y, int r){
      this.x = x;
      this.y = y;
      this.r = r;
    }
  }
}
